@@ -14,13 +14,14 @@
 
 """Hypervisor Stats action implementations"""
 
+from osc_lib.command import command
 import six
 
-from openstackclient.common import command
+from openstackclient.i18n import _
 
 
 class ShowHypervisorStats(command.ShowOne):
-    """Display hypervisor stats details"""
+    _description = _("Display hypervisor stats details")
 
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
