@@ -164,12 +164,13 @@ class CreateImage(command.ShowOne):
             type=int,
             help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--file",
             metavar="<file>",
             help=_("Upload image from local file"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--volume",
             metavar="<volume>",
             help=_("Create image from a volume"),
@@ -204,6 +205,16 @@ class CreateImage(command.ShowOne):
             action="store_true",
             help=_("Image is inaccessible to the public (default)"),
         )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
+        )
         parser.add_argument(
             "--property",
             dest="properties",
@@ -259,7 +270,7 @@ class CreateImage(command.ShowOne):
         kwargs = {}
         copy_attrs = ('name', 'id',
                       'container_format', 'disk_format',
-                      'min_disk', 'min_ram', 'tags')
+                      'min_disk', 'min_ram', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
@@ -287,7 +298,10 @@ class CreateImage(command.ShowOne):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
@@ -438,6 +452,18 @@ class ListImage(command.Lister):
             action=parseractions.KeyValueAction,
             help=_('Filter output based on property'),
         )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            default=None,
+            help=_("Filter images based on name.")
+        )
+        parser.add_argument(
+            '--status',
+            metavar='<status>',
+            default=None,
+            help=_("Filter images based on status.")
+        )
         parser.add_argument(
             '--long',
             action='store_true',
@@ -462,17 +488,17 @@ class ListImage(command.Lister):
         )
         parser.add_argument(
             "--limit",
-            metavar="<limit>",
+            metavar="<num-images>",
             type=int,
             help=_("Maximum number of images to display."),
         )
         parser.add_argument(
             '--marker',
-            metavar='<marker>',
+            metavar='<image>',
             default=None,
-            help=_("The last image (name or ID) of the previous page. Display "
+            help=_("The last image of the previous page. Display "
                    "list of images after marker. Display all images if not "
-                   "specified."),
+                   "specified. (name or ID)"),
         )
         return parser
 
@@ -491,6 +517,10 @@ class ListImage(command.Lister):
         if parsed_args.marker:
             kwargs['marker'] = utils.find_resource(image_client.images,
                                                    parsed_args.marker).id
+        if parsed_args.name:
+            kwargs['name'] = parsed_args.name
+        if parsed_args.status:
+            kwargs['status'] = parsed_args.status
         if parsed_args.long:
             columns = (
                 'ID',
@@ -697,6 +727,16 @@ class SetImage(command.Command):
             action="store_true",
             help=_("Image is inaccessible to the public (default)"),
         )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
+        )
         parser.add_argument(
             "--property",
             dest="properties",
@@ -816,7 +856,7 @@ class SetImage(command.Command):
         copy_attrs = ('architecture', 'container_format', 'disk_format',
                       'file', 'instance_id', 'kernel_id', 'locations',
                       'min_disk', 'min_ram', 'name', 'os_distro', 'os_version',
-                      'prefix', 'progress', 'ramdisk_id', 'tags')
+                      'prefix', 'progress', 'ramdisk_id', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
@@ -844,7 +884,10 @@ class SetImage(command.Command):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
@@ -904,6 +947,12 @@ class ShowImage(command.ShowOne):
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "--human-readable",
+            default=False,
+            action='store_true',
+            help=_("Print image size in a human-friendly format."),
+        )
         parser.add_argument(
             "image",
             metavar="<image>",
@@ -917,6 +966,8 @@ class ShowImage(command.ShowOne):
             image_client.images,
             parsed_args.image,
         )
+        if parsed_args.human_readable:
+            image['size'] = utils.format_size(image['size'])
 
         info = _format_image(image)
         return zip(*sorted(six.iteritems(info)))
