@@ -16,11 +16,6 @@
 import argparse
 import logging
 
-try:
-    from novaclient.v2 import security_group_rules as compute_secgroup_rules
-except ImportError:
-    from novaclient.v1_1 import security_group_rules as compute_secgroup_rules
-
 from osc_lib.cli import parseractions
 from osc_lib import exceptions
 from osc_lib import utils
@@ -51,17 +46,17 @@ def _format_network_port_range(rule):
     # - Single port: '80:80'
     # - No port range: ''
     port_range = ''
-    if _is_icmp_protocol(rule.protocol):
-        if rule.port_range_min:
-            port_range += 'type=' + str(rule.port_range_min)
-        if rule.port_range_max:
-            port_range += ':code=' + str(rule.port_range_max)
-    elif rule.port_range_min or rule.port_range_max:
-        port_range_min = str(rule.port_range_min)
-        port_range_max = str(rule.port_range_max)
-        if rule.port_range_min is None:
+    if _is_icmp_protocol(rule['protocol']):
+        if rule['port_range_min']:
+            port_range += 'type=' + str(rule['port_range_min'])
+        if rule['port_range_max']:
+            port_range += ':code=' + str(rule['port_range_max'])
+    elif rule['port_range_min'] or rule['port_range_max']:
+        port_range_min = str(rule['port_range_min'])
+        port_range_max = str(rule['port_range_max'])
+        if rule['port_range_min'] is None:
             port_range_min = port_range_max
-        if rule.port_range_max is None:
+        if rule['port_range_max'] is None:
             port_range_max = port_range_min
         port_range = port_range_min + ':' + port_range_max
     return port_range
@@ -315,7 +310,7 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
             if parsed_args.src_group:
                 LOG.warning(
                     _("The %(old)s option is deprecated, "
-                      "please use %(new)s instead.") %
+                      "please use %(new)s instead."),
                     {'old': '--src-group', 'new': '--remote-group'},
                 )
         elif not (parsed_args.remote_ip is None and
@@ -326,7 +321,7 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
             if parsed_args.src_ip:
                 LOG.warning(
                     _("The %(old)s option is deprecated, "
-                      "please use %(new)s instead.") %
+                      "please use %(new)s instead."),
                     {'old': '--src-ip', 'new': '--remote-ip'},
                 )
         elif attrs['ethertype'] == 'IPv4':
@@ -348,10 +343,7 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
         return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
-        group = utils.find_resource(
-            client.security_groups,
-            parsed_args.group,
-        )
+        group = client.api.security_group_find(parsed_args.group)
         protocol = self._get_protocol(parsed_args)
         if protocol == 'icmp':
             from_port, to_port = -1, -1
@@ -363,14 +355,13 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
         remote_ip = None
         if not (parsed_args.remote_group is None and
                 parsed_args.src_group is None):
-            parsed_args.remote_group = utils.find_resource(
-                client.security_groups,
+            parsed_args.remote_group = client.api.security_group_find(
                 parsed_args.remote_group or parsed_args.src_group,
-            ).id
+            )['id']
             if parsed_args.src_group:
                 LOG.warning(
                     _("The %(old)s option is deprecated, "
-                      "please use %(new)s instead.") %
+                      "please use %(new)s instead."),
                     {'old': '--src-group', 'new': '--remote-group'},
                 )
         if not (parsed_args.remote_ip is None and
@@ -379,20 +370,21 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
             if parsed_args.src_ip:
                 LOG.warning(
                     _("The %(old)s option is deprecated, "
-                      "please use %(new)s instead.") %
+                      "please use %(new)s instead."),
                     {'old': '--src-ip', 'new': '--remote-ip'},
                 )
         else:
             remote_ip = '0.0.0.0/0'
-        obj = client.security_group_rules.create(
-            group.id,
-            protocol,
-            from_port,
-            to_port,
-            remote_ip,
-            parsed_args.remote_group,
+
+        obj = client.api.security_group_rule_create(
+            security_group_id=group['id'],
+            ip_protocol=protocol,
+            from_port=from_port,
+            to_port=to_port,
+            remote_ip=remote_ip,
+            remote_group=parsed_args.remote_group,
         )
-        return _format_security_group_rule_show(obj._info)
+        return _format_security_group_rule_show(obj)
 
 
 class DeleteSecurityGroupRule(common.NetworkAndComputeDelete):
@@ -417,12 +409,22 @@ class DeleteSecurityGroupRule(common.NetworkAndComputeDelete):
         client.delete_security_group_rule(obj)
 
     def take_action_compute(self, client, parsed_args):
-        client.security_group_rules.delete(self.r)
+        client.api.security_group_rule_delete(self.r)
 
 
 class ListSecurityGroupRule(common.NetworkAndComputeLister):
     _description = _("List security group rules")
 
+    def _format_network_security_group_rule(self, rule):
+        """Transform the SDK SecurityGroupRule object to a dict
+
+        The SDK object gets in the way of reformatting columns...
+        Create port_range column from port_range_min and port_range_max
+        """
+        rule = rule.to_dict()
+        rule['port_range'] = _format_network_port_range(rule)
+        return rule
+
     def update_parser_common(self, parser):
         parser.add_argument(
             'group',
@@ -508,10 +510,10 @@ class ListSecurityGroupRule(common.NetworkAndComputeLister):
             'id',
             'protocol',
             'remote_ip_prefix',
-            'port_range_min',
+            'port_range',
         )
         if parsed_args.long:
-            columns = columns + ('direction', 'ethertype',)
+            columns = columns + ('direction', 'ether_type',)
         columns = columns + ('remote_group_id',)
 
         # Get the security group rules using the requested query.
@@ -535,16 +537,13 @@ class ListSecurityGroupRule(common.NetworkAndComputeLister):
         if parsed_args.protocol is not None:
             query['protocol'] = parsed_args.protocol
 
-        rules = list(client.security_group_rules(**query))
-
-        # Reformat the rules to display a port range instead
-        # of just the port range minimum. This maintains
-        # output compatibility with compute.
-        for rule in rules:
-            rule.port_range_min = _format_network_port_range(rule)
+        rules = [
+            self._format_network_security_group_rule(r)
+            for r in client.security_group_rules(**query)
+        ]
 
         return (column_headers,
-                (utils.get_item_properties(
+                (utils.get_dict_properties(
                     s, columns,
                 ) for s in rules))
 
@@ -560,27 +559,29 @@ class ListSecurityGroupRule(common.NetworkAndComputeLister):
 
         rules_to_list = []
         if parsed_args.group is not None:
-            group = utils.find_resource(
-                client.security_groups,
+            group = client.api.security_group_find(
                 parsed_args.group,
             )
-            rules_to_list = group.rules
+            rules_to_list = group['rules']
         else:
             columns = columns + ('parent_group_id',)
             search = {'all_tenants': parsed_args.all_projects}
-            for group in client.security_groups.list(search_opts=search):
-                rules_to_list.extend(group.rules)
+            for group in client.api.security_group_list(search_opts=search):
+                rules_to_list.extend(group['rules'])
 
         # NOTE(rtheis): Turn the raw rules into resources.
         rules = []
         for rule in rules_to_list:
-            rules.append(compute_secgroup_rules.SecurityGroupRule(
-                client.security_group_rules,
+            rules.append(
                 network_utils.transform_compute_security_group_rule(rule),
-            ))
+            )
+            # rules.append(compute_secgroup_rules.SecurityGroupRule(
+            #     client.security_group_rules,
+            #     network_utils.transform_compute_security_group_rule(rule),
+            # ))
 
         return (column_headers,
-                (utils.get_item_properties(
+                (utils.get_dict_properties(
                     s, columns,
                 ) for s in rules))
 
@@ -610,8 +611,8 @@ class ShowSecurityGroupRule(common.NetworkAndComputeShowOne):
         # the requested rule.
         obj = None
         security_group_rules = []
-        for security_group in client.security_groups.list():
-            security_group_rules.extend(security_group.rules)
+        for security_group in client.api.security_group_list():
+            security_group_rules.extend(security_group['rules'])
         for security_group_rule in security_group_rules:
             if parsed_args.rule == str(security_group_rule.get('id')):
                 obj = security_group_rule
