@@ -17,7 +17,7 @@ from tempest.lib import exceptions
 
 from openstackclient.tests.functional import base
 from openstackclient.tests.functional.compute.v2 import common
-from openstackclient.tests.functional.volume.v2 import test_volume
+from openstackclient.tests.functional.volume.v2 import common as volume_common
 
 
 class ServerTests(common.ComputeTestCase):
@@ -25,6 +25,7 @@ class ServerTests(common.ComputeTestCase):
 
     @classmethod
     def setUpClass(cls):
+        super(ServerTests, cls).setUpClass()
         cls.haz_network = base.is_service_enabled('network')
 
     def test_server_list(self):
@@ -282,9 +283,7 @@ class ServerTests(common.ComputeTestCase):
     def test_server_boot_from_volume(self):
         """Test server create from volume, server delete"""
         # get volume status wait function
-        volume_wait_for = test_volume.VolumeTests(
-            methodName='wait_for',
-        ).wait_for
+        volume_wait_for = volume_common.BaseVolumeTests.wait_for_status
 
         # get image size
         cmd_output = json.loads(self.openstack(
@@ -388,6 +387,113 @@ class ServerTests(common.ComputeTestCase):
             cmd_output['status'],
         )
 
+    def test_server_boot_with_bdm_snapshot(self):
+        """Test server create from image with bdm snapshot, server delete"""
+        # get volume status wait function
+        volume_wait_for = volume_common.BaseVolumeTests.wait_for_status
+        volume_wait_for_delete = volume_common.BaseVolumeTests.wait_for_delete
+
+        # create source empty volume
+        empty_volume_name = uuid.uuid4().hex
+        cmd_output = json.loads(self.openstack(
+            'volume create -f json ' +
+            '--size 1 ' +
+            empty_volume_name
+        ))
+        self.assertIsNotNone(cmd_output["id"])
+        self.addCleanup(self.openstack,
+                        'volume delete ' + empty_volume_name)
+        self.assertEqual(
+            empty_volume_name,
+            cmd_output['name'],
+        )
+        volume_wait_for("volume", empty_volume_name, "available")
+
+        # create snapshot of source empty volume
+        empty_snapshot_name = uuid.uuid4().hex
+        cmd_output = json.loads(self.openstack(
+            'volume snapshot create -f json ' +
+            '--volume ' + empty_volume_name + ' ' +
+            empty_snapshot_name
+        ))
+        self.assertIsNotNone(cmd_output["id"])
+        # Deleting volume snapshot take time, so we need to wait until the
+        # snapshot goes. Entries registered by self.addCleanup will be called
+        # in the reverse order, so we need to register wait_for_delete first.
+        self.addCleanup(volume_wait_for_delete,
+                        'volume snapshot', empty_snapshot_name)
+        self.addCleanup(self.openstack,
+                        'volume snapshot delete ' + empty_snapshot_name)
+        self.assertEqual(
+            empty_snapshot_name,
+            cmd_output['name'],
+        )
+        volume_wait_for("volume snapshot", empty_snapshot_name, "available")
+
+        # create server with bdm snapshot
+        server_name = uuid.uuid4().hex
+        server = json.loads(self.openstack(
+            'server create -f json ' +
+            '--flavor ' + self.flavor_name + ' ' +
+            '--image ' + self.image_name + ' ' +
+            '--block-device-mapping '
+            'vdb=' + empty_snapshot_name + ':snapshot:1:true ' +
+            self.network_arg + ' ' +
+            '--wait ' +
+            server_name
+        ))
+        self.assertIsNotNone(server["id"])
+        self.assertEqual(
+            server_name,
+            server['name'],
+        )
+        self.wait_for_status(server_name, 'ACTIVE')
+
+        # check server volumes_attached, format is
+        # {"volumes_attached": "id='2518bc76-bf0b-476e-ad6b-571973745bb5'",}
+        cmd_output = json.loads(self.openstack(
+            'server show -f json ' +
+            server_name
+        ))
+        volumes_attached = cmd_output['volumes_attached']
+        self.assertTrue(volumes_attached.startswith('id='))
+        attached_volume_id = volumes_attached.replace('id=', '')
+
+        # check the volume that attached on server
+        cmd_output = json.loads(self.openstack(
+            'volume show -f json ' +
+            attached_volume_id
+        ))
+        attachments = cmd_output['attachments']
+        self.assertEqual(
+            1,
+            len(attachments),
+        )
+        self.assertEqual(
+            server['id'],
+            attachments[0]['server_id'],
+        )
+        self.assertEqual(
+            "in-use",
+            cmd_output['status'],
+        )
+
+        # delete server, then check the attached volume had been deleted,
+        # <delete-on-terminate>=true
+        self.openstack('server delete --wait ' + server_name)
+        cmd_output = json.loads(self.openstack(
+            'volume list -f json'
+        ))
+        target_volume = [each_volume
+                         for each_volume in cmd_output
+                         if each_volume['ID'] == attached_volume_id]
+        if target_volume:
+            # check the attached volume is 'deleting' status
+            self.assertEqual('deleting', target_volume[0]['Status'])
+        else:
+            # the attached volume had been deleted
+            pass
+
     def test_server_create_with_none_network(self):
         """Test server create with none network option."""
         server_name = uuid.uuid4().hex
@@ -410,6 +516,52 @@ class ServerTests(common.ComputeTestCase):
         self.assertIsNotNone(server['addresses'])
         self.assertEqual('', server['addresses'])
 
+    def test_server_create_with_security_group(self):
+        """Test server create with security group ID and name"""
+        if not self.haz_network:
+            # NOTE(dtroyer): As of Ocata release Nova forces nova-network to
+            #                run in a cells v1 configuration.  Security group
+            #                and network functions currently do not work in
+            #                the gate jobs so we have to skip this.  It is
+            #                known to work tested against a Mitaka nova-net
+            #                DevStack without cells.
+            self.skipTest("No Network service present")
+        # Create two security group, use name and ID to create server
+        sg_name1 = uuid.uuid4().hex
+        security_group1 = json.loads(self.openstack(
+            'security group create -f json ' + sg_name1
+        ))
+        self.addCleanup(self.openstack, 'security group delete ' + sg_name1)
+        sg_name2 = uuid.uuid4().hex
+        security_group2 = json.loads(self.openstack(
+            'security group create -f json ' + sg_name2
+        ))
+        self.addCleanup(self.openstack, 'security group delete ' + sg_name2)
+
+        server_name = uuid.uuid4().hex
+        server = json.loads(self.openstack(
+            'server create -f json ' +
+            '--flavor ' + self.flavor_name + ' ' +
+            '--image ' + self.image_name + ' ' +
+            # Security group id is integer in nova-network, convert to string
+            '--security-group ' + str(security_group1['id']) + ' ' +
+            '--security-group ' + security_group2['name'] + ' ' +
+            self.network_arg + ' ' +
+            server_name
+        ))
+        self.addCleanup(self.openstack, 'server delete --wait ' + server_name)
+
+        self.assertIsNotNone(server['id'])
+        self.assertEqual(server_name, server['name'])
+        self.assertIn(str(security_group1['id']), server['security_groups'])
+        self.assertIn(str(security_group2['id']), server['security_groups'])
+        self.wait_for_status(server_name, 'ACTIVE')
+        server = json.loads(self.openstack(
+            'server show -f json ' + server_name
+        ))
+        self.assertIn(sg_name1, server['security_groups'])
+        self.assertIn(sg_name2, server['security_groups'])
+
     def test_server_create_with_empty_network_option_latest(self):
         """Test server create with empty network option in nova 2.latest."""
         server_name = uuid.uuid4().hex
