@@ -19,8 +19,7 @@ from openstackclient.tests.functional.network.v2 import common
 
 
 class NetworkQosPolicyTests(common.NetworkTests):
-    """Functional tests for QoS policy. """
-    NAME = uuid.uuid4().hex
+    """Functional tests for QoS policy"""
     HEADERS = ['Name']
     FIELDS = ['name']
 
@@ -28,6 +27,8 @@ class NetworkQosPolicyTests(common.NetworkTests):
     def setUpClass(cls):
         common.NetworkTests.setUpClass()
         if cls.haz_network:
+            cls.NAME = uuid.uuid4().hex
+
             opts = cls.get_opts(cls.FIELDS)
             raw_output = cls.openstack(
                 'network qos policy create ' +
@@ -38,12 +39,15 @@ class NetworkQosPolicyTests(common.NetworkTests):
 
     @classmethod
     def tearDownClass(cls):
-        if cls.haz_network:
-            raw_output = cls.openstack(
-                'network qos policy delete ' +
-                cls.NAME
-            )
-            cls.assertOutput('', raw_output)
+        try:
+            if cls.haz_network:
+                raw_output = cls.openstack(
+                    'network qos policy delete ' +
+                    cls.NAME
+                )
+                cls.assertOutput('', raw_output)
+        finally:
+            super(NetworkQosPolicyTests, cls).tearDownClass()
 
     def setUp(self):
         super(NetworkQosPolicyTests, self).setUp()
@@ -68,3 +72,16 @@ class NetworkQosPolicyTests(common.NetworkTests):
         raw_output = self.openstack('network qos policy show ' + self.NAME +
                                     opts)
         self.assertEqual("True\n", raw_output)
+
+    def test_qos_policy_default(self):
+        self.openstack('network qos policy set --default ' + self.NAME)
+        opts = self.get_opts(['is_default'])
+        raw_output = self.openstack('network qos policy show ' + self.NAME +
+                                    opts)
+        self.assertEqual("True\n", raw_output)
+
+        self.openstack('network qos policy set --no-default ' + self.NAME)
+        opts = self.get_opts(['is_default'])
+        raw_output = self.openstack('network qos policy show ' + self.NAME +
+                                    opts)
+        self.assertEqual("False\n", raw_output)
