@@ -10,14 +10,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import os
+import fixtures
 
-from openstackclient.tests.functional import base
+from openstackclient.tests.functional.volume import base
 
 
-class BaseVolumeTests(base.TestCase):
+class BaseVolumeTests(base.BaseVolumeTests):
     """Base class for Volume functional tests. """
 
-    @classmethod
-    def setUpClass(cls):
-        os.environ['OS_VOLUME_API_VERSION'] = '1'
+    def setUp(self):
+        super(BaseVolumeTests, self).setUp()
+        ver_fixture = fixtures.EnvironmentVariable(
+            'OS_VOLUME_API_VERSION', '1'
+        )
+        self.useFixture(ver_fixture)
