@@ -284,7 +284,8 @@ IDENTITY_PROVIDER = {
     'id': idp_id,
     'remote_ids': idp_remote_ids,
     'enabled': True,
-    'description': idp_description
+    'description': idp_description,
+    'domain_id': domain_id,
 }
 
 protocol_id = 'protocol'
@@ -622,6 +623,23 @@ class FakeProject(object):
                                      loaded=True)
         return project
 
+    @staticmethod
+    def create_projects(attrs=None, count=2):
+        """Create multiple fake projects.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of projects to fake
+        :return:
+            A list of FakeResource objects faking the projects
+        """
+
+        projects = []
+        for i in range(0, count):
+            projects.append(FakeProject.create_one_project(attrs))
+        return projects
+
 
 class FakeDomain(object):
     """Fake one or more domain."""
@@ -712,7 +730,7 @@ class FakeCredential(object):
             A list of FakeResource objects faking credentials
         :param Integer count:
             The number of credentials to be faked
-        :return
+        :return:
             An iterable Mock object with side_effect set to a list of faked
             credentials
         """
@@ -753,6 +771,44 @@ class FakeUser(object):
                                   loaded=True)
         return user
 
+    @staticmethod
+    def create_users(attrs=None, count=2):
+        """Create multiple fake users.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of users to fake
+        :return:
+            A list of FakeResource objects faking the users
+        """
+        users = []
+        for i in range(0, count):
+            user = FakeUser.create_one_user(attrs)
+            users.append(user)
+
+        return users
+
+    @staticmethod
+    def get_users(users=None, count=2):
+        """Get an iterable MagicMock object with a list of faked users.
+
+        If users list is provided, then initialize the Mock object with
+        the list. Otherwise create one.
+
+        :param List users:
+            A list of FakeResource objects faking users
+        :param Integer count:
+            The number of users to be faked
+        :return
+            An iterable Mock object with side_effect set to a list of faked
+            users
+        """
+        if users is None:
+            users = FakeUser.create_users(count)
+
+        return mock.Mock(side_effect=users)
+
 
 class FakeGroup(object):
     """Fake one or more group."""
@@ -812,7 +868,7 @@ class FakeGroup(object):
             A list of FakeResource objects faking groups
         :param Integer count:
             The number of groups to be faked
-        :return
+        :return:
             An iterable Mock object with side_effect set to a list of faked
             groups
         """
