@@ -19,7 +19,6 @@ from osc_lib import exceptions
 from osc_lib import utils
 
 from openstackclient.network.v2 import network
-from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
 from openstackclient.tests.unit import fakes
 from openstackclient.tests.unit.identity.v2_0 import fakes as identity_fakes_v2
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes_v3
@@ -62,18 +61,21 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         'availability_zones',
         'description',
         'id',
+        'ipv4_address_scope',
+        'ipv6_address_scope',
         'is_default',
         'name',
         'port_security_enabled',
         'project_id',
-        'provider_network_type',
-        'provider_physical_network',
-        'provider_segmentation_id',
+        'provider:network_type',
+        'provider:physical_network',
+        'provider:segmentation_id',
         'qos_policy_id',
         'router:external',
         'shared',
         'status',
         'subnets',
+        'tags',
     )
 
     data = (
@@ -82,6 +84,8 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         utils.format_list(_network.availability_zones),
         _network.description,
         _network.id,
+        _network.ipv4_address_scope_id,
+        _network.ipv6_address_scope_id,
         _network.is_default,
         _network.name,
         _network.is_port_security_enabled,
@@ -94,12 +98,14 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         _network.shared,
         _network.status,
         utils.format_list(_network.subnets),
+        utils.format_list(_network.tags),
     )
 
     def setUp(self):
         super(TestCreateNetworkIdentityV3, self).setUp()
 
         self.network.create_network = mock.Mock(return_value=self._network)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = network.CreateNetwork(self.app, self.namespace)
@@ -134,6 +140,7 @@ class TestCreateNetworkIdentityV3(TestNetwork):
             'admin_state_up': True,
             'name': self._network.name,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
@@ -181,7 +188,9 @@ class TestCreateNetworkIdentityV3(TestNetwork):
             'name': self._network.name,
             'shared': True,
             'description': self._network.description,
+            # TODO(dtroyer): Remove tenant_id when we clean up the SDK refactor
             'tenant_id': self.project.id,
+            'project_id': self.project.id,
             'is_default': True,
             'router:external': True,
             'provider:network_type': 'vlan',
@@ -221,6 +230,44 @@ class TestCreateNetworkIdentityV3(TestNetwork):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [self._network.name]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('name', self._network.name),
+            ('enable', True),
+            ('share', None),
+            ('project', None),
+            ('external', False),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_network.assert_called_once_with(
+            name=self._network.name, admin_state_up=True)
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self._network,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestCreateNetworkIdentityV2(TestNetwork):
 
@@ -236,18 +283,21 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         'availability_zones',
         'description',
         'id',
+        'ipv4_address_scope',
+        'ipv6_address_scope',
         'is_default',
         'name',
         'port_security_enabled',
         'project_id',
-        'provider_network_type',
-        'provider_physical_network',
-        'provider_segmentation_id',
+        'provider:network_type',
+        'provider:physical_network',
+        'provider:segmentation_id',
         'qos_policy_id',
         'router:external',
         'shared',
         'status',
         'subnets',
+        'tags',
     )
 
     data = (
@@ -256,6 +306,8 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         utils.format_list(_network.availability_zones),
         _network.description,
         _network.id,
+        _network.ipv4_address_scope_id,
+        _network.ipv6_address_scope_id,
         _network.is_default,
         _network.name,
         _network.is_port_security_enabled,
@@ -268,12 +320,14 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         _network.shared,
         _network.status,
         utils.format_list(_network.subnets),
+        utils.format_list(_network.tags),
     )
 
     def setUp(self):
         super(TestCreateNetworkIdentityV2, self).setUp()
 
         self.network.create_network = mock.Mock(return_value=self._network)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = network.CreateNetwork(self.app, self.namespace)
@@ -311,8 +365,11 @@ class TestCreateNetworkIdentityV2(TestNetwork):
         self.network.create_network.assert_called_once_with(**{
             'admin_state_up': True,
             'name': self._network.name,
+            # TODO(dtroyer): Remove tenant_id when we clean up the SDK refactor
             'tenant_id': self.project.id,
+            'project_id': self.project.id,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
@@ -446,6 +503,7 @@ class TestListNetwork(TestNetwork):
         'Network Type',
         'Router Type',
         'Availability Zones',
+        'Tags',
     )
 
     data = []
@@ -469,6 +527,7 @@ class TestListNetwork(TestNetwork):
             net.provider_network_type,
             network._format_router_external(net.is_router_external),
             utils.format_list(net.availability_zones),
+            utils.format_list(net.tags),
         ))
 
     def setUp(self):
@@ -479,6 +538,16 @@ class TestListNetwork(TestNetwork):
 
         self.network.networks = mock.Mock(return_value=self._network)
 
+        self._agent = \
+            network_fakes.FakeNetworkAgent.create_one_network_agent()
+        self.network.get_agent = mock.Mock(return_value=self._agent)
+
+        self.network.dhcp_agent_hosting_networks = mock.Mock(
+            return_value=self._network)
+
+        # TestListTagMixin
+        self._tag_list_resource_mock = self.network.networks
+
     def test_network_list_no_options(self):
         arglist = []
         verifylist = [
@@ -512,7 +581,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'router:external': True}
+            **{'router:external': True, 'is_router_external': True}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -529,7 +598,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'router:external': False}
+            **{'router:external': False, 'is_router_external': False}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -585,7 +654,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'admin_state_up': True}
+            **{'admin_state_up': True, 'is_admin_state_up': True}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -603,7 +672,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'admin_state_up': False}
+            **{'admin_state_up': False, 'is_admin_state_up': False}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -621,7 +690,7 @@ class TestListNetwork(TestNetwork):
 
         columns, data = self.cmd.take_action(parsed_args)
         self.network.networks.assert_called_once_with(
-            **{'tenant_id': project.id}
+            **{'tenant_id': project.id, 'project_id': project.id}
         )
 
         self.assertEqual(self.columns, columns)
@@ -640,7 +709,7 @@ class TestListNetwork(TestNetwork):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
-        filters = {'tenant_id': project.id}
+        filters = {'tenant_id': project.id, 'project_id': project.id}
 
         self.network.networks.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
@@ -658,7 +727,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'shared': True}
+            **{'shared': True, 'is_shared': True}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -675,7 +744,7 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'shared': False}
+            **{'shared': False, 'is_shared': False}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -711,7 +780,8 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'provider:network_type': network_type}
+            **{'provider:network_type': network_type,
+               'provider_network_type': network_type}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -728,7 +798,8 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'provider:physical_network': physical_network}
+            **{'provider:physical_network': physical_network,
+               'provider_physical_network': physical_network}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -745,7 +816,52 @@ class TestListNetwork(TestNetwork):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.networks.assert_called_once_with(
-            **{'provider:segmentation_id': segmentation_id}
+            **{'provider:segmentation_id': segmentation_id,
+               'provider_segmentation_id': segmentation_id}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_network_list_dhcp_agent(self):
+        arglist = [
+            '--agent', self._agent.id
+        ]
+        verifylist = [
+            ('agent_id', self._agent.id),
+        ]
+
+        attrs = {self._agent, }
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.dhcp_agent_hosting_networks.assert_called_once_with(
+            *attrs)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(list(data), list(self.data))
+
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.networks.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -754,7 +870,8 @@ class TestListNetwork(TestNetwork):
 class TestSetNetwork(TestNetwork):
 
     # The network to set.
-    _network = network_fakes.FakeNetwork.create_one_network()
+    _network = network_fakes.FakeNetwork.create_one_network(
+        {'tags': ['green', 'red']})
     qos_policy = (network_fakes.FakeNetworkQosPolicy.
                   create_one_qos_policy(attrs={'id': _network.qos_policy_id}))
 
@@ -762,6 +879,7 @@ class TestSetNetwork(TestNetwork):
         super(TestSetNetwork, self).setUp()
 
         self.network.update_network = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         self.network.find_network = mock.Mock(return_value=self._network)
         self.network.find_qos_policy = mock.Mock(return_value=self.qos_policy)
@@ -781,7 +899,6 @@ class TestSetNetwork(TestNetwork):
             '--provider-network-type', 'vlan',
             '--provider-physical-network', 'physnet1',
             '--provider-segment', '400',
-            '--no-transparent-vlan',
             '--enable-port-security',
             '--qos-policy', self.qos_policy.name,
         ]
@@ -796,7 +913,6 @@ class TestSetNetwork(TestNetwork):
             ('provider_network_type', 'vlan'),
             ('physical_network', 'physnet1'),
             ('segmentation_id', '400'),
-            ('no_transparent_vlan', True),
             ('enable_port_security', True),
             ('qos_policy', self.qos_policy.name),
         ]
@@ -814,7 +930,6 @@ class TestSetNetwork(TestNetwork):
             'provider:network_type': 'vlan',
             'provider:physical_network': 'physnet1',
             'provider:segmentation_id': '400',
-            'vlan_transparent': False,
             'port_security_enabled': True,
             'qos_policy_id': self.qos_policy.id,
         }
@@ -861,11 +976,38 @@ class TestSetNetwork(TestNetwork):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        attrs = {}
-        self.network.update_network.assert_called_once_with(
-            self._network, **attrs)
+        self.assertFalse(self.network.update_network.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._network.name)
+        verifylist.append(
+            ('network', self._network.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_network.called)
+        self.network.set_tags.assert_called_once_with(
+            self._network,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowNetwork(TestNetwork):
 
@@ -878,18 +1020,21 @@ class TestShowNetwork(TestNetwork):
         'availability_zones',
         'description',
         'id',
+        'ipv4_address_scope',
+        'ipv6_address_scope',
         'is_default',
         'name',
         'port_security_enabled',
         'project_id',
-        'provider_network_type',
-        'provider_physical_network',
-        'provider_segmentation_id',
+        'provider:network_type',
+        'provider:physical_network',
+        'provider:segmentation_id',
         'qos_policy_id',
         'router:external',
         'shared',
         'status',
         'subnets',
+        'tags',
     )
 
     data = (
@@ -898,6 +1043,8 @@ class TestShowNetwork(TestNetwork):
         utils.format_list(_network.availability_zones),
         _network.description,
         _network.id,
+        _network.ipv4_address_scope_id,
+        _network.ipv6_address_scope_id,
         _network.is_default,
         _network.name,
         _network.is_port_security_enabled,
@@ -910,6 +1057,7 @@ class TestShowNetwork(TestNetwork):
         _network.shared,
         _network.status,
         utils.format_list(_network.subnets),
+        utils.format_list(_network.tags),
     )
 
     def setUp(self):
@@ -945,372 +1093,61 @@ class TestShowNetwork(TestNetwork):
         self.assertEqual(self.data, data)
 
 
-# Tests for Nova network
-#
-class TestNetworkCompute(compute_fakes.TestComputev2):
+class TestUnsetNetwork(TestNetwork):
+
+    # The network to set.
+    _network = network_fakes.FakeNetwork.create_one_network(
+        {'tags': ['green', 'red']})
+    qos_policy = (network_fakes.FakeNetworkQosPolicy.
+                  create_one_qos_policy(attrs={'id': _network.qos_policy_id}))
 
     def setUp(self):
-        super(TestNetworkCompute, self).setUp()
+        super(TestUnsetNetwork, self).setUp()
 
-        # Get a shortcut to the compute client
-        self.compute = self.app.client_manager.compute
+        self.network.update_network = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
 
-
-class TestCreateNetworkCompute(TestNetworkCompute):
-
-    # The network to create.
-    _network = compute_fakes.FakeNetwork.create_one_network()
-
-    columns = (
-        'bridge',
-        'bridge_interface',
-        'broadcast',
-        'cidr',
-        'cidr_v6',
-        'created_at',
-        'deleted',
-        'deleted_at',
-        'dhcp_server',
-        'dhcp_start',
-        'dns1',
-        'dns2',
-        'enable_dhcp',
-        'gateway',
-        'gateway_v6',
-        'host',
-        'id',
-        'injected',
-        'label',
-        'mtu',
-        'multi_host',
-        'netmask',
-        'netmask_v6',
-        'priority',
-        'project_id',
-        'rxtx_base',
-        'share_address',
-        'updated_at',
-        'vlan',
-        'vpn_private_address',
-        'vpn_public_address',
-        'vpn_public_port',
-    )
-
-    data = (
-        _network.bridge,
-        _network.bridge_interface,
-        _network.broadcast,
-        _network.cidr,
-        _network.cidr_v6,
-        _network.created_at,
-        _network.deleted,
-        _network.deleted_at,
-        _network.dhcp_server,
-        _network.dhcp_start,
-        _network.dns1,
-        _network.dns2,
-        _network.enable_dhcp,
-        _network.gateway,
-        _network.gateway_v6,
-        _network.host,
-        _network.id,
-        _network.injected,
-        _network.label,
-        _network.mtu,
-        _network.multi_host,
-        _network.netmask,
-        _network.netmask_v6,
-        _network.priority,
-        _network.project_id,
-        _network.rxtx_base,
-        _network.share_address,
-        _network.updated_at,
-        _network.vlan,
-        _network.vpn_private_address,
-        _network.vpn_public_address,
-        _network.vpn_public_port,
-    )
-
-    def setUp(self):
-        super(TestCreateNetworkCompute, self).setUp()
-
-        self.app.client_manager.network_endpoint_enabled = False
-
-        self.compute.networks.create.return_value = self._network
+        self.network.find_network = mock.Mock(return_value=self._network)
+        self.network.find_qos_policy = mock.Mock(return_value=self.qos_policy)
 
         # Get the command object to test
-        self.cmd = network.CreateNetwork(self.app, None)
+        self.cmd = network.UnsetNetwork(self.app, self.namespace)
 
-    def test_create_no_options(self):
-        arglist = []
-        verifylist = []
+    def test_unset_nothing(self):
+        arglist = [self._network.name, ]
+        verifylist = [('network', self._network.name), ]
 
-        # Missing required args should raise exception here
-        self.assertRaises(tests_utils.ParserException, self.check_parser,
-                          self.cmd, arglist, verifylist)
-
-    def test_create_default_options(self):
-        arglist = [
-            "--subnet", self._network.cidr,
-            self._network.label,
-        ]
-        verifylist = [
-            ('subnet', self._network.cidr),
-            ('name', self._network.label),
-        ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.compute.networks.create.assert_called_once_with(**{
-            'cidr': self._network.cidr,
-            'label': self._network.label,
-        })
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
-
-
-class TestDeleteNetworkCompute(TestNetworkCompute):
-
-    def setUp(self):
-        super(TestDeleteNetworkCompute, self).setUp()
-
-        self.app.client_manager.network_endpoint_enabled = False
-
-        # The networks to delete
-        self._networks = compute_fakes.FakeNetwork.create_networks(count=3)
-
-        self.compute.networks.delete.return_value = None
-
-        # Return value of utils.find_resource()
-        self.compute.networks.get = \
-            compute_fakes.FakeNetwork.get_networks(networks=self._networks)
-
-        # Get the command object to test
-        self.cmd = network.DeleteNetwork(self.app, None)
-
-    def test_delete_one_network(self):
-        arglist = [
-            self._networks[0].label,
-        ]
-        verifylist = [
-            ('network', [self._networks[0].label]),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
         result = self.cmd.take_action(parsed_args)
 
-        self.compute.networks.delete.assert_called_once_with(
-            self._networks[0].id)
+        self.assertFalse(self.network.update_network.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
-    def test_delete_multiple_networks(self):
-        arglist = []
-        for n in self._networks:
-            arglist.append(n.label)
-        verifylist = [
-            ('network', arglist),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._network.name)
+        verifylist.append(
+            ('network', self._network.name))
 
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        calls = []
-        for n in self._networks:
-            calls.append(call(n.id))
-        self.compute.networks.delete.assert_has_calls(calls)
+        self.assertFalse(self.network.update_network.called)
+        self.network.set_tags.assert_called_once_with(
+            self._network,
+            tests_utils.CompareBySet(expected_args))
         self.assertIsNone(result)
 
-    def test_delete_multiple_networks_exception(self):
-        arglist = [
-            self._networks[0].id,
-            'xxxx-yyyy-zzzz',
-            self._networks[1].id,
-        ]
-        verifylist = [
-            ('network', arglist),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
 
-        # Fake exception in utils.find_resource()
-        # In compute v2, we use utils.find_resource() to find a network.
-        # It calls get() several times, but find() only one time. So we
-        # choose to fake get() always raise exception, then pass through.
-        # And fake find() to find the real network or not.
-        self.compute.networks.get.side_effect = Exception()
-        ret_find = [
-            self._networks[0],
-            Exception(),
-            self._networks[1],
-        ]
-        self.compute.networks.find.side_effect = ret_find
-
-        # Fake exception in delete()
-        ret_delete = [
-            None,
-            Exception(),
-        ]
-        self.compute.networks.delete = mock.Mock(side_effect=ret_delete)
-
-        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
-                          parsed_args)
-
-        # The second call of utils.find_resource() should fail. So delete()
-        # was only called twice.
-        calls = [
-            call(self._networks[0].id),
-            call(self._networks[1].id),
-        ]
-        self.compute.networks.delete.assert_has_calls(calls)
-
-
-class TestListNetworkCompute(TestNetworkCompute):
-
-    # The networks going to be listed up.
-    _networks = compute_fakes.FakeNetwork.create_networks(count=3)
-
-    columns = (
-        'ID',
-        'Name',
-        'Subnet',
-    )
-
-    data = []
-    for net in _networks:
-        data.append((
-            net.id,
-            net.label,
-            net.cidr,
-        ))
-
-    def setUp(self):
-        super(TestListNetworkCompute, self).setUp()
-
-        self.app.client_manager.network_endpoint_enabled = False
-
-        self.compute.networks.list.return_value = self._networks
-
-        # Get the command object to test
-        self.cmd = network.ListNetwork(self.app, None)
-
-    def test_network_list_no_options(self):
-        arglist = []
-        verifylist = []
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        # In base command class Lister in cliff, abstract method take_action()
-        # returns a tuple containing the column names and an iterable
-        # containing the data to be listed.
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.compute.networks.list.assert_called_once_with()
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
-
-
-class TestShowNetworkCompute(TestNetworkCompute):
-
-    # The network to show.
-    _network = compute_fakes.FakeNetwork.create_one_network()
-
-    columns = (
-        'bridge',
-        'bridge_interface',
-        'broadcast',
-        'cidr',
-        'cidr_v6',
-        'created_at',
-        'deleted',
-        'deleted_at',
-        'dhcp_server',
-        'dhcp_start',
-        'dns1',
-        'dns2',
-        'enable_dhcp',
-        'gateway',
-        'gateway_v6',
-        'host',
-        'id',
-        'injected',
-        'label',
-        'mtu',
-        'multi_host',
-        'netmask',
-        'netmask_v6',
-        'priority',
-        'project_id',
-        'rxtx_base',
-        'share_address',
-        'updated_at',
-        'vlan',
-        'vpn_private_address',
-        'vpn_public_address',
-        'vpn_public_port',
-    )
-
-    data = (
-        _network.bridge,
-        _network.bridge_interface,
-        _network.broadcast,
-        _network.cidr,
-        _network.cidr_v6,
-        _network.created_at,
-        _network.deleted,
-        _network.deleted_at,
-        _network.dhcp_server,
-        _network.dhcp_start,
-        _network.dns1,
-        _network.dns2,
-        _network.enable_dhcp,
-        _network.gateway,
-        _network.gateway_v6,
-        _network.host,
-        _network.id,
-        _network.injected,
-        _network.label,
-        _network.mtu,
-        _network.multi_host,
-        _network.netmask,
-        _network.netmask_v6,
-        _network.priority,
-        _network.project_id,
-        _network.rxtx_base,
-        _network.share_address,
-        _network.updated_at,
-        _network.vlan,
-        _network.vpn_private_address,
-        _network.vpn_public_address,
-        _network.vpn_public_port,
-    )
-
-    def setUp(self):
-        super(TestShowNetworkCompute, self).setUp()
-
-        self.app.client_manager.network_endpoint_enabled = False
-
-        # Return value of utils.find_resource()
-        self.compute.networks.get.return_value = self._network
-
-        # Get the command object to test
-        self.cmd = network.ShowNetwork(self.app, None)
-
-    def test_show_no_options(self):
-        arglist = []
-        verifylist = []
-
-        self.assertRaises(tests_utils.ParserException, self.check_parser,
-                          self.cmd, arglist, verifylist)
-
-    def test_show_all_options(self):
-        arglist = [
-            self._network.label,
-        ]
-        verifylist = [
-            ('network', self._network.label),
-        ]
-
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
