@@ -129,6 +129,7 @@ class TestCreateRouter(TestRouter):
         'project_id',
         'routes',
         'status',
+        'tags',
     )
     data = (
         router._format_admin_state(new_router.admin_state_up),
@@ -143,12 +144,14 @@ class TestCreateRouter(TestRouter):
         new_router.tenant_id,
         router._format_routes(new_router.routes),
         new_router.status,
+        osc_utils.format_list(new_router.tags),
     )
 
     def setUp(self):
         super(TestCreateRouter, self).setUp()
 
         self.network.create_router = mock.Mock(return_value=self.new_router)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = router.CreateRouter(self.app, self.namespace)
@@ -159,6 +162,7 @@ class TestCreateRouter(TestRouter):
 
         self.assertRaises(tests_utils.ParserException, self.check_parser,
                           self.cmd, arglist, verifylist)
+        self.assertFalse(self.network.set_tags.called)
 
     def test_create_default_options(self):
         arglist = [
@@ -178,6 +182,7 @@ class TestCreateRouter(TestRouter):
             'admin_state_up': True,
             'name': self.new_router.name,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
@@ -265,6 +270,45 @@ class TestCreateRouter(TestRouter):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [self.new_router.name]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('name', self.new_router.name),
+            ('enable', True),
+            ('distributed', False),
+            ('ha', False),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_router.assert_called_once_with(
+            name=self.new_router.name,
+            admin_state_up=True
+        )
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self.new_router,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestDeleteRouter(TestRouter):
 
@@ -363,11 +407,13 @@ class TestListRouter(TestRouter):
     columns_long = columns + (
         'Routes',
         'External gateway info',
-        'Availability zones'
+        'Availability zones',
+        'Tags',
     )
     columns_long_no_az = columns + (
         'Routes',
         'External gateway info',
+        'Tags',
     )
 
     data = []
@@ -381,6 +427,21 @@ class TestListRouter(TestRouter):
             r.ha,
             r.tenant_id,
         ))
+
+    router_agent_data = []
+    for r in routers:
+        router_agent_data.append((
+            r.id,
+            r.name,
+            r.external_gateway_info,
+        ))
+
+    agents_columns = (
+        'ID',
+        'Name',
+        'External Gateway Info',
+    )
+
     data_long = []
     for i in range(0, len(routers)):
         r = routers[i]
@@ -389,6 +450,7 @@ class TestListRouter(TestRouter):
                 router._format_routes(r.routes),
                 router._format_external_gateway_info(r.external_gateway_info),
                 osc_utils.format_list(r.availability_zones),
+                osc_utils.format_list(r.tags),
             )
         )
     data_long_no_az = []
@@ -398,6 +460,7 @@ class TestListRouter(TestRouter):
             data[i] + (
                 router._format_routes(r.routes),
                 router._format_external_gateway_info(r.external_gateway_info),
+                osc_utils.format_list(r.tags),
             )
         )
 
@@ -407,8 +470,15 @@ class TestListRouter(TestRouter):
         # Get the command object to test
         self.cmd = router.ListRouter(self.app, self.namespace)
 
+        self.network.agent_hosted_routers = mock.Mock(
+            return_value=self.routers)
         self.network.routers = mock.Mock(return_value=self.routers)
         self.network.find_extension = mock.Mock(return_value=self._extensions)
+        self.network.find_router = mock.Mock(return_value=self.routers[0])
+        self._testagent = \
+            network_fakes.FakeNetworkAgent.create_one_network_agent()
+        self.network.get_agent = mock.Mock(return_value=self._testagent)
+        self.network.get_router = mock.Mock(return_value=self.routers[0])
 
     def test_router_list_no_options(self):
         arglist = []
@@ -556,6 +626,59 @@ class TestListRouter(TestRouter):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_router_list_agents_no_args(self):
+        arglist = [
+            '--agents',
+        ]
+        verifylist = []
+
+        # Missing required router ID should bail here
+        self.assertRaises(tests_utils.ParserException, self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+    def test_router_list_agents(self):
+        arglist = [
+            '--agent', self._testagent.id,
+        ]
+        verifylist = [
+            ('agent', self._testagent.id),
+        ]
+
+        attrs = {self._testagent.id, }
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.agent_hosted_routers(
+            *attrs)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.routers.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestRemovePortFromRouter(TestRouter):
     '''Remove port from a Router '''
@@ -641,13 +764,15 @@ class TestSetRouter(TestRouter):
     _network = network_fakes.FakeNetwork.create_one_network()
     _subnet = network_fakes.FakeSubnet.create_one_subnet()
     _router = network_fakes.FakeRouter.create_one_router(
-        attrs={'routes': [_default_route]}
+        attrs={'routes': [_default_route],
+               'tags': ['green', 'red']}
     )
 
     def setUp(self):
         super(TestSetRouter, self).setUp()
         self.network.router_add_gateway = mock.Mock()
         self.network.update_router = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         self.network.find_router = mock.Mock(return_value=self._router)
         self.network.find_network = mock.Mock(return_value=self._network)
         self.network.find_subnet = mock.Mock(return_value=self._subnet)
@@ -852,9 +977,8 @@ class TestSetRouter(TestRouter):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        attrs = {}
-        self.network.update_router.assert_called_once_with(
-            self._router, **attrs)
+        self.assertFalse(self.network.update_router.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
     def test_wrong_gateway_params(self):
@@ -961,6 +1085,34 @@ class TestSetRouter(TestRouter):
                 'enable_snat': True, }})
         self.assertIsNone(result)
 
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._router.name)
+        verifylist.append(
+            ('router', self._router.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_router.called)
+        self.network.set_tags.assert_called_once_with(
+            self._router,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowRouter(TestRouter):
 
@@ -980,6 +1132,7 @@ class TestShowRouter(TestRouter):
         'project_id',
         'routes',
         'status',
+        'tags',
     )
     data = (
         router._format_admin_state(_router.admin_state_up),
@@ -994,6 +1147,7 @@ class TestShowRouter(TestRouter):
         _router.tenant_id,
         router._format_routes(_router.routes),
         _router.status,
+        osc_utils.format_list(_router.tags),
     )
 
     def setUp(self):
@@ -1036,10 +1190,12 @@ class TestUnsetRouter(TestRouter):
             {'routes': [{"destination": "192.168.101.1/24",
                          "nexthop": "172.24.4.3"},
                         {"destination": "192.168.101.2/24",
-                         "nexthop": "172.24.4.3"}], })
+                         "nexthop": "172.24.4.3"}],
+             'tags': ['green', 'red'], })
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet()
         self.network.find_router = mock.Mock(return_value=self._testrouter)
         self.network.update_router = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         # Get the command object to test
         self.cmd = router.UnsetRouter(self.app, self.namespace)
 
@@ -1090,3 +1246,31 @@ class TestUnsetRouter(TestRouter):
         self.network.update_router.assert_called_once_with(
             self._testrouter, **attrs)
         self.assertIsNone(result)
+
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._testrouter.name)
+        verifylist.append(
+            ('router', self._testrouter.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_router.called)
+        self.network.set_tags.assert_called_once_with(
+            self._testrouter,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
+
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
