@@ -125,6 +125,7 @@ class TestCreateSubnet(TestSubnet):
         'segment_id',
         'service_types',
         'subnetpool_id',
+        'tags',
     )
 
     data = (
@@ -145,6 +146,7 @@ class TestCreateSubnet(TestSubnet):
         _subnet.segment_id,
         utils.format_list(_subnet.service_types),
         _subnet.subnetpool_id,
+        utils.format_list(_subnet.tags),
     )
 
     data_subnet_pool = (
@@ -165,6 +167,7 @@ class TestCreateSubnet(TestSubnet):
         _subnet_from_pool.segment_id,
         utils.format_list(_subnet_from_pool.service_types),
         _subnet_from_pool.subnetpool_id,
+        utils.format_list(_subnet.tags),
     )
 
     data_ipv6 = (
@@ -185,6 +188,7 @@ class TestCreateSubnet(TestSubnet):
         _subnet_ipv6.segment_id,
         utils.format_list(_subnet_ipv6.service_types),
         _subnet_ipv6.subnetpool_id,
+        utils.format_list(_subnet.tags),
     )
 
     def setUp(self):
@@ -197,6 +201,8 @@ class TestCreateSubnet(TestSubnet):
         self.domains_mock.get.return_value = self.domain
 
         # Mock SDK calls for all tests.
+        self.network.create_subnet = mock.Mock(return_value=self._subnet)
+        self.network.set_tags = mock.Mock(return_value=None)
         self.network.find_network = mock.Mock(return_value=self._network)
         self.network.find_segment = mock.Mock(
             return_value=self._network_segment
@@ -213,10 +219,11 @@ class TestCreateSubnet(TestSubnet):
         # throw a "ParserExecption"
         self.assertRaises(tests_utils.ParserException,
                           self.check_parser, self.cmd, arglist, verifylist)
+        self.assertFalse(self.network.create_subnet.called)
+        self.assertFalse(self.network.set_tags.called)
 
     def test_create_default_options(self):
         # Mock SDK calls for this test.
-        self.network.create_subnet = mock.Mock(return_value=self._subnet)
         self._network.id = self._subnet.network_id
 
         arglist = [
@@ -230,7 +237,6 @@ class TestCreateSubnet(TestSubnet):
             ('network', self._subnet.network_id),
             ('ip_version', self._subnet.ip_version),
             ('gateway', 'auto'),
-
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -242,13 +248,14 @@ class TestCreateSubnet(TestSubnet):
             'name': self._subnet.name,
             'network_id': self._subnet.network_id,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
     def test_create_from_subnet_pool_options(self):
         # Mock SDK calls for this test.
-        self.network.create_subnet = \
-            mock.Mock(return_value=self._subnet_from_pool)
+        self.network.create_subnet.return_value = self._subnet_from_pool
+        self.network.set_tags = mock.Mock(return_value=None)
         self._network.id = self._subnet_from_pool.network_id
 
         arglist = [
@@ -309,7 +316,7 @@ class TestCreateSubnet(TestSubnet):
 
     def test_create_options_subnet_range_ipv6(self):
         # Mock SDK calls for this test.
-        self.network.create_subnet = mock.Mock(return_value=self._subnet_ipv6)
+        self.network.create_subnet.return_value = self._subnet_ipv6
         self._network.id = self._subnet_ipv6.network_id
 
         arglist = [
@@ -376,12 +383,12 @@ class TestCreateSubnet(TestSubnet):
             'allocation_pools': self._subnet_ipv6.allocation_pools,
             'service_types': self._subnet_ipv6.service_types,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data_ipv6, data)
 
     def test_create_with_network_segment(self):
         # Mock SDK calls for this test.
-        self.network.create_subnet = mock.Mock(return_value=self._subnet)
         self._network.id = self._subnet.network_id
 
         arglist = [
@@ -410,12 +417,12 @@ class TestCreateSubnet(TestSubnet):
             'network_id': self._subnet.network_id,
             'segment_id': self._network_segment.id,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
     def test_create_with_description(self):
         # Mock SDK calls for this test.
-        self.network.create_subnet = mock.Mock(return_value=self._subnet)
         self._network.id = self._subnet.network_id
 
         arglist = [
@@ -444,9 +451,55 @@ class TestCreateSubnet(TestSubnet):
             'network_id': self._subnet.network_id,
             'description': self._subnet.description,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [
+            "--subnet-range", self._subnet.cidr,
+            "--network", self._subnet.network_id,
+            self._subnet.name,
+        ]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('name', self._subnet.name),
+            ('subnet_range', self._subnet.cidr),
+            ('network', self._subnet.network_id),
+            ('ip_version', self._subnet.ip_version),
+            ('gateway', 'auto'),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_subnet.assert_called_once_with(
+            cidr=self._subnet.cidr,
+            ip_version=self._subnet.ip_version,
+            name=self._subnet.name,
+            network_id=self._subnet.network_id)
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self._subnet,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestDeleteSubnet(TestSubnet):
 
@@ -546,6 +599,7 @@ class TestListSubnet(TestSubnet):
         'IP Version',
         'Gateway',
         'Service Types',
+        'Tags',
     )
 
     data = []
@@ -572,6 +626,7 @@ class TestListSubnet(TestSubnet):
             subnet.ip_version,
             subnet.gateway_ip,
             utils.format_list(subnet.service_types),
+            utils.format_list(subnet.tags),
         ))
 
     def setUp(self):
@@ -801,14 +856,41 @@ class TestListSubnet(TestSubnet):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.subnets.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestSetSubnet(TestSubnet):
 
-    _subnet = network_fakes.FakeSubnet.create_one_subnet()
+    _subnet = network_fakes.FakeSubnet.create_one_subnet(
+        {'tags': ['green', 'red']})
 
     def setUp(self):
         super(TestSetSubnet, self).setUp()
         self.network.update_subnet = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         self.network.find_subnet = mock.Mock(return_value=self._subnet)
         self.cmd = subnet_v2.SetSubnet(self.app, self.namespace)
 
@@ -867,8 +949,8 @@ class TestSetSubnet(TestSubnet):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        attrs = {}
-        self.network.update_subnet.assert_called_with(self._subnet, **attrs)
+        self.assertFalse(self.network.update_subnet.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
     def test_append_options(self):
@@ -959,6 +1041,34 @@ class TestSetSubnet(TestSubnet):
             _testsubnet, **attrs)
         self.assertIsNone(result)
 
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._subnet.name)
+        verifylist.append(
+            ('subnet', self._subnet.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_subnet.called)
+        self.network.set_tags.assert_called_once_with(
+            self._subnet,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowSubnet(TestSubnet):
     # The subnets to be shown
@@ -982,6 +1092,7 @@ class TestShowSubnet(TestSubnet):
         'segment_id',
         'service_types',
         'subnetpool_id',
+        'tags',
     )
 
     data = (
@@ -1002,6 +1113,7 @@ class TestShowSubnet(TestSubnet):
         _subnet.segment_id,
         utils.format_list(_subnet.service_types),
         _subnet.subnetpool_id,
+        utils.format_list(_subnet.tags),
     )
 
     def setUp(self):
@@ -1055,9 +1167,11 @@ class TestUnsetSubnet(TestSubnet):
                                   {'start': '8.8.8.160',
                                    'end': '8.8.8.170'}],
              'service_types': ['network:router_gateway',
-                               'network:floatingip_agent_gateway'], })
+                               'network:floatingip_agent_gateway'],
+             'tags': ['green', 'red'], })
         self.network.find_subnet = mock.Mock(return_value=self._testsubnet)
         self.network.update_subnet = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         # Get the command object to test
         self.cmd = subnet_v2.UnsetSubnet(self.app, self.namespace)
 
@@ -1169,3 +1283,31 @@ class TestUnsetSubnet(TestSubnet):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action, parsed_args)
+
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._testsubnet.name)
+        verifylist.append(
+            ('subnet', self._testsubnet.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_subnet.called)
+        self.network.set_tags.assert_called_once_with(
+            self._testsubnet,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
+
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
