@@ -60,6 +60,7 @@ class TestCreateSubnetPool(TestSubnetPool):
         'prefixes',
         'project_id',
         'shared',
+        'tags',
     )
     data = (
         _subnet_pool.address_scope_id,
@@ -75,6 +76,7 @@ class TestCreateSubnetPool(TestSubnetPool):
         utils.format_list(_subnet_pool.prefixes),
         _subnet_pool.project_id,
         _subnet_pool.shared,
+        utils.format_list(_subnet_pool.tags),
     )
 
     def setUp(self):
@@ -82,6 +84,7 @@ class TestCreateSubnetPool(TestSubnetPool):
 
         self.network.create_subnet_pool = mock.Mock(
             return_value=self._subnet_pool)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = subnet_pool.CreateSubnetPool(self.app, self.namespace)
@@ -98,6 +101,7 @@ class TestCreateSubnetPool(TestSubnetPool):
 
         self.assertRaises(tests_utils.ParserException, self.check_parser,
                           self.cmd, arglist, verifylist)
+        self.assertFalse(self.network.set_tags.called)
 
     def test_create_no_pool_prefix(self):
         """Make sure --pool-prefix is a required argument"""
@@ -127,6 +131,7 @@ class TestCreateSubnetPool(TestSubnetPool):
             'prefixes': ['10.0.10.0/24'],
             'name': self._subnet_pool.name,
         })
+        self.assertFalse(self.network.set_tags.called)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
@@ -270,6 +275,67 @@ class TestCreateSubnetPool(TestSubnetPool):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def test_create_with_default_quota(self):
+        arglist = [
+            '--pool-prefix', '10.0.10.0/24',
+            '--default-quota', '10',
+            self._subnet_pool.name,
+        ]
+        verifylist = [
+            ('prefixes', ['10.0.10.0/24']),
+            ('default_quota', 10),
+            ('name', self._subnet_pool.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+        self.network.create_subnet_pool.assert_called_once_with(**{
+            'name': self._subnet_pool.name,
+            'prefixes': ['10.0.10.0/24'],
+            'default_quota': 10,
+        })
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [
+            '--pool-prefix', '10.0.10.0/24',
+            self._subnet_pool.name,
+        ]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('prefixes', ['10.0.10.0/24']),
+            ('name', self._subnet_pool.name),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_subnet_pool.assert_called_once_with(
+            prefixes=['10.0.10.0/24'],
+            name=self._subnet_pool.name
+        )
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self._subnet_pool,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestDeleteSubnetPool(TestSubnetPool):
 
@@ -367,6 +433,7 @@ class TestListSubnetPool(TestSubnetPool):
         'Address Scope',
         'Default Subnet Pool',
         'Shared',
+        'Tags',
     )
 
     data = []
@@ -387,6 +454,7 @@ class TestListSubnetPool(TestSubnetPool):
             pool.address_scope_id,
             pool.is_default,
             pool.shared,
+            utils.format_list(pool.tags),
         ))
 
     def setUp(self):
@@ -435,7 +503,7 @@ class TestListSubnetPool(TestSubnetPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
-        filters = {'shared': False}
+        filters = {'shared': False, 'is_shared': False}
 
         self.network.subnet_pools.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
@@ -451,7 +519,7 @@ class TestListSubnetPool(TestSubnetPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
-        filters = {'shared': True}
+        filters = {'shared': True, 'is_shared': True}
 
         self.network.subnet_pools.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
@@ -501,7 +569,7 @@ class TestListSubnetPool(TestSubnetPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
-        filters = {'tenant_id': project.id}
+        filters = {'tenant_id': project.id, 'project_id': project.id}
 
         self.network.subnet_pools.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
@@ -521,7 +589,7 @@ class TestListSubnetPool(TestSubnetPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
-        filters = {'tenant_id': project.id}
+        filters = {'tenant_id': project.id, 'project_id': project.id}
 
         self.network.subnet_pools.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
@@ -563,11 +631,39 @@ class TestListSubnetPool(TestSubnetPool):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.subnet_pools.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestSetSubnetPool(TestSubnetPool):
 
     # The subnet_pool to set.
-    _subnet_pool = network_fakes.FakeSubnetPool.create_one_subnet_pool()
+    _subnet_pool = network_fakes.FakeSubnetPool.create_one_subnet_pool(
+        {'default_quota': 10,
+         'tags': ['green', 'red']}
+    )
 
     _address_scope = network_fakes.FakeAddressScope.create_one_address_scope()
 
@@ -575,6 +671,7 @@ class TestSetSubnetPool(TestSubnetPool):
         super(TestSetSubnetPool, self).setUp()
 
         self.network.update_subnet_pool = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         self.network.find_subnet_pool = mock.Mock(
             return_value=self._subnet_pool)
@@ -644,9 +741,8 @@ class TestSetSubnetPool(TestSubnetPool):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        attrs = {}
-        self.network.update_subnet_pool.assert_called_once_with(
-            self._subnet_pool, **attrs)
+        self.assertFalse(self.network.update_subnet_pool.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
     def test_set_len_negative(self):
@@ -794,6 +890,51 @@ class TestSetSubnetPool(TestSubnetPool):
             self._subnet_pool, **attrs)
         self.assertIsNone(result)
 
+    def test_set_with_default_quota(self):
+        arglist = [
+            '--default-quota', '20',
+            self._subnet_pool.name,
+        ]
+        verifylist = [
+            ('default_quota', 20),
+            ('subnet_pool', self._subnet_pool.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_subnet_pool.assert_called_once_with(
+            self._subnet_pool,
+            **{'default_quota': 20, }
+        )
+        self.assertIsNone(result)
+
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._subnet_pool.name)
+        verifylist.append(
+            ('subnet_pool', self._subnet_pool.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_subnet_pool.called)
+        self.network.set_tags.assert_called_once_with(
+            self._subnet_pool,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowSubnetPool(TestSubnetPool):
 
@@ -814,6 +955,7 @@ class TestShowSubnetPool(TestSubnetPool):
         'prefixes',
         'project_id',
         'shared',
+        'tags',
     )
 
     data = (
@@ -830,6 +972,7 @@ class TestShowSubnetPool(TestSubnetPool):
         utils.format_list(_subnet_pool.prefixes),
         _subnet_pool.tenant_id,
         _subnet_pool.shared,
+        utils.format_list(_subnet_pool.tags),
     )
 
     def setUp(self):
@@ -874,10 +1017,12 @@ class TestUnsetSubnetPool(TestSubnetPool):
         super(TestUnsetSubnetPool, self).setUp()
         self._subnetpool = network_fakes.FakeSubnetPool.create_one_subnet_pool(
             {'prefixes': ['10.0.10.0/24', '10.1.10.0/24',
-                          '10.2.10.0/24'], })
+                          '10.2.10.0/24'],
+             'tags': ['green', 'red']})
         self.network.find_subnet_pool = mock.Mock(
             return_value=self._subnetpool)
         self.network.update_subnet_pool = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         # Get the command object to test
         self.cmd = subnet_pool.UnsetSubnetPool(self.app, self.namespace)
 
@@ -887,7 +1032,10 @@ class TestUnsetSubnetPool(TestSubnetPool):
             '--pool-prefix', '10.1.10.0/24',
             self._subnetpool.name,
         ]
-        verifylist = [('prefixes', ['10.0.10.0/24', '10.1.10.0/24'])]
+        verifylist = [
+            ('prefixes', ['10.0.10.0/24', '10.1.10.0/24']),
+            ('subnet_pool', self._subnetpool.name),
+        ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
         attrs = {'prefixes': ['10.2.10.0/24']}
@@ -900,8 +1048,39 @@ class TestUnsetSubnetPool(TestSubnetPool):
             '--pool-prefix', '10.100.1.1/25',
             self._subnetpool.name,
         ]
-        verifylist = [('prefixes', ['10.100.1.1/25'])]
+        verifylist = [
+            ('prefixes', ['10.100.1.1/25']),
+            ('subnet_pool', self._subnetpool.name),
+        ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action,
                           parsed_args)
+
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._subnetpool.name)
+        verifylist.append(
+            ('subnet_pool', self._subnetpool.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_subnet_pool.called)
+        self.network.set_tags.assert_called_once_with(
+            self._subnetpool,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
+
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
